#!/usr/bin/env python3

# Turns a Meson testlog.json file into a JUnit XML report
#
# Copyright 2019  GNOME Foundation
#
# SPDX-License-Identifier: LGPL-2.1-or-later
#
# Original author: Emmanuele Bassi

import argparse
import datetime
import json
import sys
import xml.etree.ElementTree as ET

aparser = argparse.ArgumentParser(
    description="Turns a Meson test log into a JUnit report"
)
aparser.add_argument(
    "--project-name", metavar="NAME", help="The project name", default="unknown"
)
aparser.add_argument(
    "--job-id", metavar="ID", help="The job ID for the report", default="Unknown"
)
aparser.add_argument(
    "--branch",
    metavar="NAME",
    help="Branch of the project being tested",
    default="main",
)
aparser.add_argument(
    "--output",
    metavar="FILE",
    help="The output file, stdout by default",
    type=argparse.FileType("w", encoding="UTF-8"),
    default=sys.stdout,
)
aparser.add_argument(
    "infile",
    metavar="FILE",
    help="The input testlog.json, stdin by default",
    type=argparse.FileType("r", encoding="UTF-8"),
    default=sys.stdin,
)

args = aparser.parse_args()

outfile = args.output

testsuites = ET.Element("testsuites")
testsuites.set("id", "{}/{}".format(args.job_id, args.branch))
testsuites.set("package", args.project_name)
testsuites.set("timestamp", datetime.datetime.utcnow().isoformat())

suites = {}
for line in args.infile:
    data = json.loads(line)
    (full_suite, unit_name) = data["name"].split(" / ")
    try:
        (project_name, suite_name) = full_suite.split(":")
    except ValueError:
        project_name = full_suite
        suite_name = full_suite

    duration = data["duration"]
    return_code = data["returncode"]
    log = data["stdout"]
    log_stderr = data.get("stderr", "")

    unit = {
        "suite": suite_name,
        "name": unit_name,
        "duration": duration,
        "returncode": return_code,
        "stdout": log,
        "stderr": log_stderr,
    }

    units = suites.setdefault(suite_name, [])
    units.append(unit)

for name, units in suites.items():
    print("Processing suite {} (units: {})".format(name, len(units)))

    def if_failed(unit):
        if unit["returncode"] != 0:
            return True
        return False

    def if_succeded(unit):
        if unit["returncode"] == 0:
            return True
        return False

    successes = list(filter(if_succeded, units))
    failures = list(filter(if_failed, units))
    print(" - {}: {} pass, {} fail".format(name, len(successes), len(failures)))

    testsuite = ET.SubElement(testsuites, "testsuite")
    testsuite.set("name", "{}/{}".format(args.project_name, name))
    testsuite.set("tests", str(len(units)))
    testsuite.set("errors", str(len(failures)))
    testsuite.set("failures", str(len(failures)))

    for unit in successes:
        testcase = ET.SubElement(testsuite, "testcase")
        testcase.set("classname", "{}/{}".format(args.project_name, unit["suite"]))
        testcase.set("name", unit["name"])
        testcase.set("time", str(unit["duration"]))

    for unit in failures:
        testcase = ET.SubElement(testsuite, "testcase")
        testcase.set("classname", "{}/{}".format(args.project_name, unit["suite"]))
        testcase.set("name", unit["name"])
        testcase.set("time", str(unit["duration"]))

        failure = ET.SubElement(testcase, "failure")
        failure.set("classname", "{}/{}".format(args.project_name, unit["suite"]))
        failure.set("name", unit["name"])
        failure.set("type", "error")
        failure.text = unit["stdout"] + "\n" + unit["stderr"]

output = ET.tostring(testsuites, encoding="unicode")
outfile.write(output)



#!/usr/bin/env python3
# Copyright (C) 1998, 1999 Tom Tromey
# Copyright (C) 2001 Red Hat Software
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, see <http://www.gnu.org/licenses/>.

"""
gen-casefold-txt.py - Generate test cases for casefolding from Unicode data.
See http://www.unicode.org/Public/UNIDATA/UnicodeCharacterDatabase.html
Usage:
    I consider the output of this program to be unrestricted.
    Use it as you will.
"""

import sys
import argparse


def main(argv):
    parser = argparse.ArgumentParser(
        description="Generate test cases for casefolding from Unicode data"
    )
    parser.add_argument("UNICODE-VERSION")
    parser.add_argument("CaseFolding.txt")
    args = parser.parse_args(argv[1:])
    version = getattr(args, "UNICODE-VERSION")
    filename = getattr(args, "CaseFolding.txt")

    print(
        """\
# Test cases generated from Unicode {} data
# by gen-casefold-txt.py. Do not edit.
#
# Some special hand crafted tests
#
AaBbCc@@\taabbcc@@
#
# Now the automatic tests
#""".format(
            version
        )
    )

    # Names of fields in the CaseFolding table
    CODE, STATUS, MAPPING = range(3)

    with open(filename, encoding="utf-8") as fileobj:
        for line in fileobj:
            # strip comments and skip empty lines
            line = line.split("#", 1)[0].strip()
            if not line:
                continue

            fields = [f.strip() for f in line.split(";", 3)[:3]]
            if len(fields) != 3:
                raise SystemExit(
                    "Entry for %s has wrong number of fields (%d)"
                    % (fields[CODE], len(fields))
                )

            status = fields[STATUS]
            # skip simple and Turkic mappings
            if status in "ST":
                continue

            code = chr(int(fields[CODE], 16))
            values = "".join([chr(int(v, 16)) for v in fields[MAPPING].split()])
            print("{}\t{}".format(code, values))


if __name__ == "__main__":
    sys.exit(main(sys.argv))
